@@ -7,7 +7,7 @@
 //! is up to the application to provide an interface, and to configure
 //! the policy appropriately.
 //!
-//! [`StandardPolicy`]: https://docs.rs/sequoia-openpgp/1.10.0/sequoia_openpgp/policy/struct.StandardPolicy.html
+//! [`StandardPolicy`]: https://docs.rs/sequoia-openpgp/1.*/sequoia_openpgp/policy/struct.StandardPolicy.html
 //!
 //! This library provides a high-level interface that parses a
 //! configuration file, and returns a configured `StandardPolicy`.
@@ -267,9 +267,9 @@
 //!
 //! signature.v3 = 2017-01-01
 //! signature.v4 = 2030-01-01
-//! signature.v5 = "always"
-//! # v5 signatures are coming, but not yet recognized.
-//! signature.ignore_invalid = "v5"
+//! signature.v6 = "always"
+//! # v6 signatures are coming, but not yet recognized.
+//! signature.ignore_invalid = "v6"
 //! ```
 //!
 //! ## Examples
@@ -296,6 +296,8 @@
 //! # Ok(()) }
 //! ```
 
+#![allow(clippy::type_complexity)]
+
 use std::collections::HashSet;
 use std::env;
 use std::io;
@@ -314,7 +316,6 @@ use openpgp::types::AEADAlgorithm;
 use openpgp::types::HashAlgorithm;
 use openpgp::types::SymmetricAlgorithm;
 
-use anyhow;
 use anyhow::Context as _;
 
 use chrono::DateTime;
@@ -324,9 +325,12 @@ use chrono::Utc;
 use toml::value;
 use toml::Value;
 
+mod dump;
+pub use dump::DumpDefault;
 #[cfg(test)]
 mod testdata;
 
+#[derive(Clone, Debug)]
 pub struct ConfiguredStandardPolicy<'a> {
     policy: StandardPolicy<'a>,
 }
@@ -347,6 +351,10 @@ pub enum Error {
     /// A Relative Path was provided where an absolute path was expected.
     #[error("Relative path not allowed: {0}")]
     RelativePathError(PathBuf),
+
+    /// An algorithm is not known to this crate.
+    #[error("Unknown algorithm: {0}")]
+    UnknownAlgorithm(String),
 }
 
 /// A timestamp.
@@ -381,7 +389,7 @@ fn parse_time(t: &Value) -> Result<Timestamp> {
             let t = if let Ok(p) = DateTime::parse_from_rfc3339(t) {
                 p.into()
             } else {
-                let t = NaiveDate::parse_from_str(&t, "%Y-%m-%d")
+                let t = NaiveDate::parse_from_str(t, "%Y-%m-%d")
                     .context(format!("Parsing timestamp {}", t))?
                     .and_hms_opt(0, 0, 0).expect("valid time");
                 let t: DateTime<Utc> = DateTime::from_utc(t, Utc);
@@ -400,37 +408,36 @@ fn check_sections(path: Option<&str>,
                   section: &value::Map<String, Value>,
                   known_keys: &[&str]) -> Result<()> {
     // known_keys better be lowercase.
-    known_keys.iter().for_each(|&s| assert_eq!(&s.to_lowercase()[..], s));
+    known_keys.iter().for_each(
+        |&s| debug_assert_eq!(&s.to_lowercase()[..], s));
 
-    let prefix = if let Some(path) = path {
+    let prefix = || if let Some(path) = path {
         format!("{}.", path)
     } else {
         "".to_string()
     };
 
-    let keys: HashSet<String> = section
+    let keys: HashSet<&str> = section
         .keys()
-        .cloned()
+        .map(|s| s.as_str())
         .collect();
 
     // The set of allowed keys are the known keys, plus
     // "ignore_invalid", and the value of "ignore_invalid".
-    let mut allowed_keys: Vec<String> = known_keys.iter()
-        .map(|k| k.to_string())
-        .collect();
+    let mut allowed_keys: Vec<&str> = known_keys.to_vec();
     if let Some(ignore) = section.get("ignore_invalid") {
-        allowed_keys.push("ignore_invalid".to_string());
+        allowed_keys.push("ignore_invalid");
         match ignore {
-            Value::String(k) => allowed_keys.push(k.clone()),
+            Value::String(k) => allowed_keys.push(k.as_str()),
             Value::Array(ks) => {
                 for k in ks {
                     if let Value::String(k) = k {
-                        allowed_keys.push(k.clone());
+                        allowed_keys.push(k.as_str());
                     } else {
                         Err(Error::ParseError(format!(
                             "'{}ignore_invalid' takes a string \
                              or an array of strings",
-                            prefix)))?
+                            prefix())))?
                     }
                 }
             }
@@ -438,15 +445,15 @@ fn check_sections(path: Option<&str>,
                 return Err(Error::ParseError(format!(
                     "Invalid value for '{}ignore_invalid': {}, \
                      expected a string or an array of strings",
-                    prefix, ignore)).into());
+                    prefix(), ignore)).into());
             }
         }
     }
 
     // Now check if there are any unknown sections.
     let unknown_keys = keys
-        .difference(&HashSet::<String>::from_iter(allowed_keys.into_iter()))
-        .cloned()
+        .difference(&allowed_keys.into_iter().collect())
+        .map(|s| *s)
         .collect::<Vec<_>>();
     if ! unknown_keys.is_empty() {
         return Err(Error::ParseError(format!(
@@ -479,7 +486,7 @@ const HASH_ALGO_MAP: [(&str, HashAlgorithm, &[(&str, HashAlgoSecurity)]); 7] = [
     ("sha224", HashAlgorithm::SHA224, HASH_ALGO_PROPS)
 ];
 
-const ASYMM_ALGO_MAP: [(&str, AsymmetricAlgorithm, &[(&str, ())]); 18] = [
+const ASYMM_ALGO_MAP: [(&str, AsymmetricAlgorithm, &[(&str, ())]); 19] = [
     ("rsa1024", AsymmetricAlgorithm::RSA1024, &[]),
     ("rsa2048", AsymmetricAlgorithm::RSA2048, &[]),
     ("rsa3072", AsymmetricAlgorithm::RSA3072, &[]),
@@ -496,6 +503,7 @@ const ASYMM_ALGO_MAP: [(&str, AsymmetricAlgorithm, &[(&str, ())]); 18] = [
     ("nistp384", AsymmetricAlgorithm::NistP384, &[]),
     ("nistp521", AsymmetricAlgorithm::NistP521, &[]),
     ("brainpoolp256", AsymmetricAlgorithm::BrainpoolP256, &[]),
+    ("brainpoolp384", AsymmetricAlgorithm::BrainpoolP384, &[]),
     ("brainpoolp512", AsymmetricAlgorithm::BrainpoolP512, &[]),
     ("cv25519", AsymmetricAlgorithm::Cv25519, &[]),
 ];
@@ -514,9 +522,10 @@ const SYMM_ALGO_MAP: [(&str, SymmetricAlgorithm, &[(&str, ())]); 11] = [
     ("camellia256", SymmetricAlgorithm::Camellia256, &[]),
 ];
 
-const AEAD_ALGO_MAP: [(&str, AEADAlgorithm, &[(&str, ())]); 2] = [
+const AEAD_ALGO_MAP: [(&str, AEADAlgorithm, &[(&str, ())]); 3] = [
     ("eax", AEADAlgorithm::EAX, &[]),
     ("ocb", AEADAlgorithm::OCB, &[]),
+    ("gcm", AEADAlgorithm::GCM, &[]),
 ];
 
 const PACKET_MAP: [(&str, Tag, &[(&str, u8)]); 18] = [
@@ -540,6 +549,45 @@ const PACKET_MAP: [(&str, Tag, &[(&str, u8)]); 18] = [
     ("aed", Tag::AED, &[("v1", 1)]),
 ];
 
+/// Given a map and an algorithm (or packet tag), returns the key
+/// usable in the configuration file.
+fn algo_to_key<A, V>(map: &[(&'static str, A, &[V])], a: A)
+                     -> std::result::Result<&'static str, Error>
+where
+    A: Copy + PartialEq + ToString,
+{
+    map.iter().find_map(|e| (e.1 == a).then_some(e.0))
+        .ok_or_else(|| Error::UnknownAlgorithm(a.to_string()))
+}
+
+/// Given a packet tag, returns whether `v` is a known version.
+fn known_packet_version(t: Tag, v: u8) -> bool {
+    PACKET_MAP.iter().find(|e| e.1 == t)
+        .map(|(_, _, versions)| versions.iter().any(|(_, known)| *known == v))
+        .unwrap_or(false)
+}
+
+/// Returns whether the given packet type is versioned.
+fn versioned_packet(t: Tag) -> bool {
+    matches!(t,
+             | Tag::PKESK
+             | Tag::Signature
+             | Tag::SKESK
+             | Tag::OnePassSig
+             | Tag::SecretKey
+             | Tag::PublicKey
+             | Tag::SecretSubkey
+             | Tag::PublicSubkey
+             | Tag::SEIP
+             | Tag::AED)
+}
+
+impl Default for ConfiguredStandardPolicy<'_> {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 impl<'a> ConfiguredStandardPolicy<'a> {
     /// The default environment variable.
     pub const ENV_VAR: &'static str
@@ -751,7 +799,7 @@ impl<'a> ConfiguredStandardPolicy<'a> {
             config
         } else {
             return Err(Error::ParseError(
-                format!("Expected toml sections")).into());
+                "Expected toml sections".into()).into());
         };
 
         check_sections(
@@ -1020,6 +1068,92 @@ mod tests {
     use openpgp::Cert;
     use openpgp::parse::Parse;
 
+    use quickcheck::{Arbitrary, Gen};
+
+    fn arbitrary_cutoff(g: &mut Gen) -> Option<SystemTime> {
+        match g.choose(&[0, 1, 2]).unwrap() {
+            0 => None,
+            1 => Some(UNIX_EPOCH),
+            2 => Some(SystemTime::arbitrary(g)),
+            _ => unreachable!(),
+        }
+    }
+
+    impl Arbitrary for ConfiguredStandardPolicy<'static> {
+        fn arbitrary(g: &mut Gen) -> Self {
+            let mut p = ConfiguredStandardPolicy::default();
+
+            for _ in 0..u8::arbitrary(g) {
+                let cutoff = arbitrary_cutoff(g);
+
+                match g.choose(&[0, 1, 2, 3, 4]).unwrap() {
+                    // Hash algorithms.
+                    0 => {
+                        let variants =
+                            HashAlgorithm::variants().collect::<Vec<_>>();
+                        let a = g.choose(&variants).unwrap();
+                        if let Some(sec) = g.choose(&[
+                            Some(HashAlgoSecurity::SecondPreImageResistance),
+                            Some(HashAlgoSecurity::CollisionResistance),
+                            None,
+                        ]).unwrap() {
+                            p.policy.reject_hash_property_at(*a, *sec, cutoff);
+                        } else {
+                            p.policy.reject_hash_at(*a, cutoff);
+                        }
+                    },
+
+                    // Symmetric algorithms.
+                    1 => {
+                        let variants =
+                            SymmetricAlgorithm::variants().collect::<Vec<_>>();
+                        let a = g.choose(&variants).unwrap();
+                        p.policy.reject_symmetric_algo_at(*a, cutoff);
+                    },
+
+                    // Asymmetric algorithms.
+                    2 => {
+                        let variants =
+                            AsymmetricAlgorithm::variants().collect::<Vec<_>>();
+                        let a = g.choose(&variants).unwrap();
+                        p.policy.reject_asymmetric_algo_at(*a, cutoff);
+                    },
+
+                    // AEAD algorithms.
+                    3 => {
+                        let variants =
+                            AEADAlgorithm::variants().collect::<Vec<_>>();
+                        let a = g.choose(&variants).unwrap();
+                        p.policy.reject_aead_algo_at(*a, cutoff);
+                    },
+
+                    // Packets.
+                    4 => {
+                        let variants = Tag::variants().collect::<Vec<_>>();
+                        let t = g.choose(&variants).unwrap();
+
+                        if versioned_packet(*t) && bool::arbitrary(g) {
+                            let versions =
+                                PACKET_MAP.iter().find(|e| e.1 == *t).unwrap()
+                                .2.iter().map(|(_, v)| *v)
+                                .collect::<Vec<_>>();
+                            let v = g.choose(&versions).unwrap();
+
+                            p.policy.reject_packet_tag_version_at(
+                                *t, *v, cutoff);
+                        } else {
+                            p.policy.reject_packet_tag_at(*t, cutoff);
+                        }
+                    },
+
+                    _ => unreachable!(),
+                }
+            }
+
+            p
+        }
+    }
+
     // Check that invalid sections cause an error, unless they are
     // explicitly ignored.
     #[test]
